library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity capturer is
	port (	clk		: in	std_logic;
		reset		: in	std_logic;

		cam_clk		: in	std_logic;
		cam_href	: in	std_logic;
		cam_vsync	: in	std_logic;
		cam_y		: in	std_logic_vector(7 downto 0);
		cam_uv		: in	std_logic_vector(7 downto 0);

		pause		: in	std_logic;
		dout		: out	std_logic_vector(7 downto 0);
		doute		: out	std_logic;
		sync		: out	std_logic
		);
end capturer;

architecture arh of capturer is

	constant W : integer := 352;
	constant H : integer := 288;
	constant W_len : integer := 12;

	signal wrbank, rdbank : std_logic := '0';
	signal wrofs, rdofs, xrdofs : unsigned(W_len-1 downto 0);
	signal addrr, addrw : unsigned(W_len downto 0);

	signal cam_done, cam_first_row, start, first_row : std_logic := '0';

	type rdstate_t is (idle, send_sync, code_y, code_u, code_v, flush);
	signal rdstate : rdstate_t;

--	signal rdstatec : integer;

	signal rdlin_x : unsigned(W_len-1 downto 0);
	signal rdsegid : integer range 0 to W/8-1;
	signal rdx, rdy : integer range 0 to 7;
	signal plane_W : integer range W/2 to W;

	signal doutmuxy, doutmuxuv : std_logic_vector(7 downto 0);

begin

--	with rdstate select
--		rdstatec <=	0 when idle,
--				1 when send_sync,
--				2 when code_y,
--				3 when code_u,
--				4 when code_v,
--				5 when flush,
--				-1 when others;

	sync <= '1' when rdstate = send_sync else '0';

	with rdstate select
		dout <= doutmuxy when code_y,
		        doutmuxuv when code_u,
		        doutmuxuv when code_v,
		        doutmuxuv when flush,
		        (others => 'X') when others;

	with rdstate select
		plane_W <= W when code_y,
		           W/2 when others;

	rdx <= to_integer(to_01(rdlin_x(2 downto 0)));		-- linearno kretanje lin_x pretvori u po 8x8 segmentima
	rdy <= to_integer(to_01(rdlin_x(5 downto 3)));
	rdsegid <= to_integer(to_01(rdlin_x(W_len-1 downto 6)));
	xrdofs <= to_unsigned((rdy*plane_W) + (rdsegid*8) + rdx, W_len);

	with rdstate select
		rdofs <= xrdofs when idle,
		         xrdofs when send_sync,
		         xrdofs when code_y,
		         xrdofs(W_len-2 downto 0) & "0" when code_u,
		         xrdofs(W_len-2 downto 0) & "1" when code_v,
		         (others => 'X') when others;

	rdbank <= not wrbank;
	addrw <= wrbank & wrofs;
	addrr <= rdbank & rdofs;
	mem_y: entity work.memory
		generic map (bits => 8)
		port map (
			clkr => clk,
			clkw => cam_clk,
			din => cam_y,
			wr => cam_href,
			addrw => addrw,
			addrr => addrr,
			dout => doutmuxy
		);
	mem_uv: entity work.memory
		generic map (bits => 4)
		port map (
			clkr => clk,
			clkw => cam_clk,
			din => cam_uv(7 downto 4),
			wr => cam_href,
			addrw => addrw,
			addrr => addrr,
			dout => doutmuxuv(7 downto 4)
		);
	doutmuxuv(3 downto 0) <= (others => '0');

	writer: process (cam_clk)
	begin
		if rising_edge(cam_clk) then
			cam_done <= '0';
			if cam_vsync = '1' then
				cam_first_row <= '1';
				wrofs <= to_unsigned(0, 12);
			elsif cam_href = '1' then
				if wrofs = (W*8)-1 then		-- gotov redak
					first_row <= cam_first_row;
					cam_first_row <= '0';
					cam_done <= '1';
					wrofs <= to_unsigned(0, 12);
					wrbank <= not wrbank;
				else
					wrofs <= wrofs + 1;
				end if;
			end if;
		end if;
	end process;

	syncer: entity work.pulsesync port map (cam_clk, cam_done, clk, start);

	reader: process (clk)
	begin
		if rising_edge(clk) then
			doute <= '1';
			case rdstate is
				when idle =>
					doute <= '0';
					rdlin_x <= to_unsigned(0, W_len);
					if start = '1' then
						if first_row = '1' then
							rdlin_x <= to_unsigned(0, W_len);
							rdstate <= send_sync;
						else
							rdlin_x <= to_unsigned(1, W_len);
							rdstate <= code_y;
							doute <= '1';
						end if;
					else
						rdstate <= idle;
					end if;

				when send_sync =>
					rdlin_x <= to_unsigned(1, W_len);
					rdstate <= code_y;

				when code_y =>
					if pause = '0' then
						if rdlin_x = W*8-1 then
							rdstate <= code_u;
							rdlin_x <= to_unsigned(0, W_len);
						else
							rdlin_x <= rdlin_x + 1;
						end if;
					else
						rdlin_x <= rdlin_x;
						rdstate <= code_y;
						doute <= '0';
					end if;

				when code_u =>
					if pause = '0' then
						if rdlin_x = W*4-1 then
							rdstate <= code_v;
							rdlin_x <= to_unsigned(0, W_len);
						else
							rdlin_x <= rdlin_x + 1;
						end if;
					else
						rdlin_x <= rdlin_x;
						rdstate <= code_u;
						doute <= '0';
					end if;

				when code_v =>
					if pause = '0' then
						if rdlin_x = W*4-1 then
							rdstate <= flush;
							rdlin_x <= (others => 'X');
						else
							rdlin_x <= rdlin_x + 1;
						end if;
					else
						rdlin_x <= rdlin_x;
						rdstate <= code_v;
						doute <= '0';
					end if;

				when flush =>
					doute <= '0';
					rdlin_x <= (others => 'X');
					rdstate <= idle;

				when others =>
					doute <= '0';
					rdlin_x <= (others => 'X');
					rdstate <= idle;
			end case;

			if reset = '1' then
				doute <= '0';
				rdstate <= idle;
				rdlin_x <= to_unsigned(0, W_len);
			end if;
		end if;
	end process;

end arh;
