library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity quantizer is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		dine	: in	std_logic;
		din	: in	signed(7 downto 0);
		doute	: out	std_logic;
		dout	: out	std_logic_vector(7 downto 0)
		);
end quantizer;

architecture arh of quantizer is

	signal collectbuf, sendbuf : std_logic_vector(0 to 127);
	signal coefid : integer range 0 to 63;

	signal roundw : unsigned(2 downto 0);
	signal rounder : signed(7 downto 0);
	signal rounded : std_logic_vector(7 downto 0);

	constant nbytes : integer := sendbuf'length / 8;
	signal start, sending : std_logic;
	signal sendid : integer range 0 to nbytes - 1;

begin

	sending <= '1' when start='1' or sendid /= 0 else '0';
	doute <= sending;
	dout <= sendbuf(sendid*8 to sendid*8+7);

	sender: process (clk)
	begin
		if rising_edge(clk) then
			if reset = '1' then
				sendid <= 0;
			elsif sending = '1' then
				if sendid = nbytes-1 then
					sendid <= 0;
				else
					sendid <= sendid + 1;
				end if;
			end if;
		end if;
	end process;


	process (roundw, din)
	begin
		rounder <= (others => '0');
		rounder(to_integer(roundw)) <= din(to_integer(roundw));
	end process;

	rounded <= std_logic_vector(din + rounder);

	quantizator: process (clk)
	begin
		if rising_edge(clk) then
			start <= '0';
			if reset = '1' then
				coefid <= 0;
			elsif dine = '1' then
				case coefid is
					when 0 =>
						collectbuf(  0 to   3) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 1 =>
						collectbuf(  4 to   7) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 2 =>
						collectbuf(  8 to  12) <= rounded(7 downto 3);
						roundw <= o"2";	-- w=5
					when 3 =>
						collectbuf( 13 to  16) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 4 =>
						collectbuf( 17 to  20) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 5 =>
						collectbuf( 21 to  24) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 6 =>
						collectbuf( 25 to  28) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 7 =>
						collectbuf( 29 to  32) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 8 =>
						collectbuf( 33 to  36) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 9 =>
						collectbuf( 37 to  40) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 10 =>
						collectbuf( 41 to  43) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 11 =>
						collectbuf( 44 to  47) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 12 =>
						collectbuf( 48 to  51) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 13 =>
						collectbuf( 52 to  55) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 14 =>
						collectbuf( 56 to  58) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 15 =>
						collectbuf( 59 to  61) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 16 =>
						collectbuf( 62 to  64) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 17 =>
						collectbuf( 65 to  67) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 18 =>
						collectbuf( 68 to  70) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 19 =>
						collectbuf( 71 to  73) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 20 =>
						collectbuf( 74 to  76) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 21 =>
						collectbuf( 77 to  78) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 22 =>
						collectbuf( 79 to  80) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 23 =>
						collectbuf( 81 to  83) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 24 =>
						collectbuf( 84 to  86) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 25 =>
						collectbuf( 87 to  89) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 26 =>
						collectbuf( 90 to  92) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 27 =>
						collectbuf( 93 to  94) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 28 =>
						collectbuf( 95 to  95) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 29 =>
						collectbuf( 96 to  97) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 30 =>
						collectbuf( 98 to  99) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 31 =>
						collectbuf(100 to 101) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 32 =>
						collectbuf(102 to 103) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 33 =>
						collectbuf(104 to 105) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 34 =>
						collectbuf(106 to 107) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 35 =>
						collectbuf(108 to 108) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 36 =>
						collectbuf(109 to 109) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 37 =>
						collectbuf(110 to 110) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 38 =>
						collectbuf(111 to 112) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 39 =>
						collectbuf(113 to 114) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 40 =>
						collectbuf(115 to 116) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 41 =>
						collectbuf(117 to 117) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 42 =>
						collectbuf(118 to 118) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 43 =>
						collectbuf(119 to 119) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 44 =>
						collectbuf(120 to 120) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 45 =>
						collectbuf(121 to 121) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 46 =>
						collectbuf(122 to 122) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 47 =>
						collectbuf(123 to 123) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 48 =>
						collectbuf(124 to 124) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 49 =>
						collectbuf(125 to 125) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 50 =>
						collectbuf(126 to 126) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 51 =>
						collectbuf(127 to 127) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when others => null;
				end case;
				if coefid = 63 then
					coefid <= 0;
					sendbuf <= collectbuf;
					start <= '1';
				else
					coefid <= coefid + 1;
				end if;
			end if;
		end if;
	end process;

end arh;
